
import time
from django.test import TestCase
from .decorators import enable


class AfterResponseTest(TestCase):

    def test_execution(self):
        self.executed = 0

        @enable
        def func(val):
            if val:
                self.executed += 1

        self.assertEqual(self.executed, 0)
        self.client.get('/')
        time.sleep(0.1)
        self.assertEqual(self.executed, 0)

        func.after_response(True)

        self.assertEqual(self.executed, 0)
        self.client.get('/')
        time.sleep(0.1)
        self.assertEqual(self.executed, 1)

        func.after_response(True)
        func.after_response(True)

        self.assertEqual(self.executed, 1)
        self.client.get('/')
        time.sleep(0.1)
        self.assertEqual(self.executed, 3)

        func.after_response(False)
        func.after_response(True)

        self.assertEqual(self.executed, 3)
        self.client.get('/')
        time.sleep(0.1)
        self.assertEqual(self.executed, 4)

    def test_slow_execution(self):
        self.executed = 0

        @enable
        def slow_func():
            time.sleep(1)
            self.executed += 1

        slow_func.after_response()

        self.assertEqual(self.executed, 0)
        self.client.get('/')
        self.assertEqual(self.executed, 0)

        slow_func.after_response()

        before_time = time.time()
        self.client.get('/')
        after_time = time.time()
        self.assertTrue(after_time - before_time < 0.5)




from __future__ import unicode_literals, print_function
import logging
import threading

from django.core.signals import request_finished
from django.conf import settings

from .store import function_queue

logger = logging.getLogger(__name__)


AFTER_RESPONSE_RUN_ASYNC = getattr(settings, 'AFTER_RESPONSE_RUN_ASYNC', True)


def run(func, *args, **kwargs):
    try:
        func(*args, **kwargs)
    except Exception as e:
        logger.exception(str(e))


def run_em_all(sender, **kwargs):
    while len(function_queue):
        func, args, kwargs = function_queue.pop()
        if AFTER_RESPONSE_RUN_ASYNC:
            threading.Thread(target=run, args=(func,) + args, kwargs=kwargs).start()
        else:
            run(func, *args, **kwargs)
request_finished.connect(run_em_all)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
import re


with open('after_response/__init__.py', 'r') as init_file:
    version = re.search(
        '^__version__ = [\'"]([^\'"]+)[\'"]',
        init_file.read(),
        re.MULTILINE,
    ).group(1)


setup(
    name='django-after-response',
    description='Simple asynchronous execution',
    url='http://github.com/defrex/django-after-response/',
    license='MIT',
    author='Aron Jones',
    author_email='aron.jones@gmail.com',
    packages=['after_response'],
    version=version,
    install_requires=[
        'Django>=1.5',
    ],
)




from django.conf import settings
from .store import function_queue


AFTER_RESPONSE_IMMEDIATE = getattr(settings, 'AFTER_RESPONSE_IMMEDIATE', False)


def enable(func):
    def after_response(*args, **kwargs):
        if AFTER_RESPONSE_IMMEDIATE:
            func(*args, **kwargs)
        else:
            function_queue.append((func, args, kwargs))
    func.after_response = after_response
    return func
